function P_bat = SOC2Pbat(SOC,C_bat,eff_ch,eff_di)

% Converts absolute SOC from the SDP to a SOC (next period) and p_bat (this
% period)

%% Parameter explaination
% SOC has the dimensions (N_days,N_periods)
% C_bat is the useable capacity of the battery

%% Initialisation
N_days = size(SOC,1);
N_periods = size(SOC,2);
P_bat = zeros(N_days,N_periods);


%% Calculations
SOC_t = SOC(1,1);
for day = 1:N_days
    for t = 1:(N_periods-1)
        SOC_tp1 = SOC(day,t+1);

        P_bat_temp = (SOC_tp1-SOC_t)*C_bat;
        
        if P_bat_temp > 0
            P_bat(day,t) = P_bat_temp/eff_ch;  % eff_ch < 0
        else
            P_bat(day,t) = P_bat_temp*eff_di;  % eff_di < 0
        end
        
        SOC_t = SOC_tp1;
    end
    if day < N_days
        SOC_tp1 = SOC(day+1,1);
        
        P_bat_temp = (SOC_tp1-SOC_t)*C_bat;
        
        if P_bat_temp > 0
            P_bat(day,N_periods) = P_bat_temp/eff_ch;  % eff_ch < 0
        else
            P_bat(day,N_periods) = P_bat_temp*eff_di;  % eff_di < 0
        end
        
    end
end

end