
% clear
% clc
% 
loadnum     = 1;
structure   = 4;
opt_days    = 30;
pv          = 0;
N_disc      = 20;
c_multi     = 1;
p_multi     = 1;
learning    = 0;

firstday = 5; % First day of 2016 was a Friday.


fprintf('- - - - - - - - - - - - - - - - - - - - - \n');
fprintf('Optimization software v0.2 \n');
fprintf('Written by Jrgen Srgrd Erdal \n');
fprintf('- - - - - - - - - - - - - - - - - - - - - \n\n');

fprintf('Load number: \t \t %i \n',loadnum);
fprintf('Utility structure: \t %i \n',structure);
fprintf('Optimization horizon: \t %i days \n',opt_days);
fprintf('Load discretization: \t %i levels \n',N_disc);

%% Battery data
[eff_ch,eff_di,C_bat,P_bat_nom,SOC_max,SOC_min] = batteryData(c_multi,p_multi); % This could be a input as a struct.

N_soc = 100;
delta_soc = 1/N_soc; % For optimiser

SOC_initial = 0;
SOC_final = 0;

soc_initial = SOC2socnum(SOC_initial,SOC_max,SOC_min,N_soc);
soc_final = SOC2socnum(SOC_final,SOC_max,SOC_min,N_soc);

%% PV panel data

[P_peak, A, NOCT, tempcoeff, eta_sys, eta_module, eta_total] = pvpanelData();

%% Loading of data
load2013 = loadread(2013,loadnum);
load2014 = loadread(2014,loadnum);
load2015 = loadread(2015,loadnum);
load2016 = loadread(2016,loadnum);

load = {load2013, load2014, load2015, load2016};

spot2013 = priceread(2013);
spot2014 = priceread(2014);
spot2015 = priceread(2015);
spot2016 = priceread(2016);

% spot = {spot2013, spot2014, spot2015, spot2016};

N_periods = size(load2016,2);

% Calculation of PPV
if pv == 1
    irradiance = LMTread(2016,1);

    temp = LMTread(2016,2);

    Tcell = tcell(irradiance,temp,NOCT);

    ppv = ppvadvanced(P_peak,eta_sys,irradiance,tempcoeff,Tcell);     % Advanced calculations

    fprintf('PV: \t \t \t Yes \n');
    fprintf('\n \n');
    
else
    ppv = zeros(opt_days,N_periods); % No production. Used to model grid + battery (i.e. no PV)
    
    fprintf('PV: \t \t \t No \n');
    fprintf('\n \n');
end

%% Creating daydata

daydata = cell(1,3);

for year = 2013:2015

    daydata{year-2012} = daydataCreator(load{year-2012},year);
    
end

%% Initial trans_matrix calculation

% Add all mon-fri data and sat-sun on top of each other
weekdays = dataAgg(daydata{1},[1 2 3 4 5]);
weekends = dataAgg(daydata{1},[6 7]);

for year = 2:3 % 1 = 2013, 2 = 2014 and 3 = 2015

    weekdaytemp = dataAgg(daydata{year},[1 2 3 4 5]);
    weekendtemp = dataAgg(daydata{year},[6 7]);
    weekdays = [weekdays ; weekdaytemp];
    weekends = [weekends ; weekendtemp];
    
end

% Run them through dataDistribution to get two separate trans_matrix sets
[wd_distribution, wd_trans_matrix, wd_disc] = dataDistribution(weekdays,N_disc);
[we_distribution, we_trans_matrix, we_disc] = dataDistribution(weekends,N_disc);



%% Loop starts here
daynum = firstday; % First day of year
P_bat_sdp = zeros(opt_days,N_periods);
SOC_sdp = zeros(opt_days,N_periods);
utilityprice = getUtilityPrice(2016,structure);

dp_timer = zeros(1,10);

for day = 1:10
    
    loadvec = load2016(day,:);
    pricevec = spot2016(day,:);
    ppvvec = ppv(day,:);

    [totalcost, cost_opt, path_opt, P_bat_opt, SOC_opt,time] = ...
        optimumCharge(loadvec,ppvvec,pricevec,utilityprice,structure,P_bat_nom,eff_ch,eff_di,C_bat,SOC_max,SOC_min,delta_soc,firstday);
    
    dp_timer(day) = time;

    [P_bat_opt, SOC_opt] = pbatConversion(loadvec,P_bat_opt,SOC_opt,SOC_initial,SOC_min);


end

mean(dp_timer)