function [distribution, trans_matrix, disc] = dataDistribution(data,N_disc)

% This function takes in a data matrix of dimension (days, time) and an int 
% discretization defining the number of discrete steps desired. It returns 
% the data divided into its discretized steps.

%% Step 1: Checking for N_disc >= 2
if N_disc < 2
    error('N_disc has to be greater than 1')
end

%% Step 2: Initialization
N_periods = size(data,2);
N_days = size(data,1);

disc = zeros(N_disc,size(data,2));

distribution = zeros(N_disc,N_periods,N_disc);
trans_matrix = zeros(N_disc,N_periods,N_disc);

%% Step 3: Setting disc
disc(1,:) = max(data);
disc(N_disc,:) = min(data);

for i = 1:N_periods
    
    disc(:,i) = linspace(disc(1,i),disc(N_disc,i),N_disc);
    
end

disc = round(disc,1);

%% Step 4: Counting occurrences
for day = 1:N_days
    for t = 1:(N_periods-1)
        
        temp_t = abs(disc(:,t)-data(day,t));
        temp_tp1 = abs(disc(:,t+1)-data(day,t+1));
        
        [~, idx_t] = min(temp_t);
        [~, idx_tp1] = min(temp_tp1);
        
        distribution(idx_t,t,idx_tp1) = distribution(idx_t,t,idx_tp1) + 1;
        
    end
end

%% Calculating probabilities
for t = 1:N_periods
    for state = 1:N_disc
        
        statecount = sum(distribution(state,t,:));
        
        if statecount ~= 0
            trans_matrix(state,t,:) = distribution(state,t,:)./statecount;
        end
        
    end
end

end