function bill_vec = master_main(loadnum,structure,opt_days,pv,N_disc,c_multi,p_multi,learning)

% clear
% clc
% 
% loadnum     = 1;
% structure   = 3;
% opt_days    = 30;
% pv          = 0;

firstday = 5; % First day of 2016 was a Friday.

% N_disc = 20;

fprintf('- - - - - - - - - - - - - - - - - - - - - \n');
fprintf('Optimization software v0.2 \n');
fprintf('Written by Jrgen Srgrd Erdal \n');
fprintf('- - - - - - - - - - - - - - - - - - - - - \n\n');

fprintf('Load number: \t \t %i \n',loadnum);
fprintf('Utility structure: \t %i \n',structure);
fprintf('Optimization horizon: \t %i days \n',opt_days);
fprintf('Load discretization: \t %i levels \n',N_disc);

%% Battery data
[eff_ch,eff_di,C_bat,P_bat_nom,SOC_max,SOC_min] = batteryData(c_multi,p_multi); % This could be a input as a struct.

N_soc = 100;
delta_soc = 1/N_soc; % For optimiser

SOC_initial = 0;
SOC_final = 0;

soc_initial = SOC2socnum(SOC_initial,SOC_max,SOC_min,N_soc);
soc_final = SOC2socnum(SOC_final,SOC_max,SOC_min,N_soc);

%% PV panel data

[P_peak, A, NOCT, tempcoeff, eta_sys, eta_module, eta_total] = pvpanelData();

%% Loading of data
load2013 = loadread(2013,loadnum);
load2014 = loadread(2014,loadnum);
load2015 = loadread(2015,loadnum);
load2016 = loadread(2016,loadnum);

load = {load2013, load2014, load2015, load2016};

spot2013 = priceread(2013);
spot2014 = priceread(2014);
spot2015 = priceread(2015);
spot2016 = priceread(2016);

% spot = {spot2013, spot2014, spot2015, spot2016};

N_periods = size(load2016,2);

% Calculation of PPV
if pv == 1
    irradiance = LMTread(2016,1);

    temp = LMTread(2016,2);

    Tcell = tcell(irradiance,temp,NOCT);

    ppv = ppvadvanced(P_peak,eta_sys,irradiance,tempcoeff,Tcell);     % Advanced calculations

    fprintf('PV: \t \t \t Yes \n');
    fprintf('\n \n');
    
else
    ppv = zeros(opt_days,N_periods); % No production. Used to model grid + battery (i.e. no PV)
    
    fprintf('PV: \t \t \t No \n');
    fprintf('\n \n');
end

%% Creating daydata

daydata = cell(1,3);

for year = 2013:2015

    daydata{year-2012} = daydataCreator(load{year-2012},year);
    
end

%% Initial trans_matrix calculation

% Add all mon-fri data and sat-sun on top of each other
weekdays = dataAgg(daydata{1},[1 2 3 4 5]);
weekends = dataAgg(daydata{1},[6 7]);

for year = 2:3 % 1 = 2013, 2 = 2014 and 3 = 2015

    weekdaytemp = dataAgg(daydata{year},[1 2 3 4 5]);
    weekendtemp = dataAgg(daydata{year},[6 7]);
    weekdays = [weekdays ; weekdaytemp];
    weekends = [weekends ; weekendtemp];
    
end

% Run them through dataDistribution to get two separate trans_matrix sets
[wd_distribution, wd_trans_matrix, wd_disc] = dataDistribution(weekdays,N_disc);
[we_distribution, we_trans_matrix, we_disc] = dataDistribution(weekends,N_disc);



%% Loop starts here
daynum = firstday; % First day of year
P_bat_sdp = zeros(opt_days,N_periods);
SOC_sdp = zeros(opt_days,N_periods);
utilityprice = getUtilityPrice(2016,structure);

% load_ind = zeros(opt_days,N_periods);
% trans_prob = zeros(opt_days,N_periods);
% trans_matrix_final = zeros(opt_days,N_periods,N_disc);

for day = 1:opt_days
    
    fprintf('Day number %i: ',day);
    
    %% Fetching data for current day
    pricevec = spot2016(day,:);
    loadvec = load2016(day,:);
    ppvvec = ppv(day,:);
    
    %% Optimization of battery strategy
    
    if daynum <= 5
        % Run weekday
        path_fSDP = fixedHorizonSDP(wd_disc,wd_trans_matrix,ppvvec,...
            pricevec,utilityprice,structure,P_bat_nom,eff_ch,eff_di,...
            C_bat,SOC_max,SOC_min,N_soc,SOC_initial,SOC_final,daynum);
    else
        % Run weekend
        path_fSDP = fixedHorizonSDP(we_disc,we_trans_matrix,ppvvec,...
            pricevec,utilityprice,structure,P_bat_nom,eff_ch,eff_di,...
            C_bat,SOC_max,SOC_min,N_soc,SOC_initial,SOC_final,daynum);
    end
    
%     sdp_timer(it) = time;
%     it = it + 1;
%     
%     if it == 11
%         sdp_timer_mean = mean(sdp_timer);
%         % stop
%     end
    
    %% Period (hour) iteration
    
    soc_t = zeros(1,N_periods);
    soc_t(1) = soc_initial;
    soc_t(N_periods) = soc_final;
    
    for t = 1:(N_periods-1)

        %% Evaluate load and decide SOC in next time step
        if daynum <= 5
            
            loadtemp_t = abs(wd_disc(:,t)-loadvec(t));

            [~, idx_t] = min(loadtemp_t);
            
            soc_t(t+1) = path_fSDP(soc_t(t),t,idx_t);
            
%             load_ind(day,t) = idx_t;
%             trans_prob(day,t) = trans_matrix_final(day,t,idx_t);
%             trans_matrix_final(day,t+1,:) = wd_trans_matrix(idx_t,t,:);
        else
            
            loadtemp_t = abs(we_disc(:,t)-loadvec(t));

            [~, idx_t] = min(loadtemp_t);
            
            soc_t(t+1) = path_fSDP(soc_t(t),t,idx_t);
            
%             load_ind(day,t) = idx_t;
%             trans_prob(day,t) = trans_matrix_final(day,t,idx_t);
%             trans_matrix_final(day,t+1,:) = we_trans_matrix(idx_t,t,:);
        end % if

    end % for t 
    
%     if daynum < 6
%         loadtemp_t = abs(wd_disc(:,24)-loadvec(24));
%     else
%         loadtemp_t = abs(we_disc(:,24)-loadvec(24));
%     end
% 
%     [~, idx_t] = min(loadtemp_t);
%     
%     load_ind(day,24) = idx_t;
%     trans_prob(day,24) = trans_matrix_final(day,24,idx_t);

    %% Calculate P_bat
    SOC_sdp(day,:) = socnum2SOC(soc_t,SOC_max,SOC_min,N_soc);
    P_bat_sdp(day,:) = socnum2Pbat(soc_t,C_bat,eff_ch,eff_di,N_soc);
    
    %% Update trans_matrices
    if learning == 1
        if daynum < 6
            weekdays = [weekdays; loadvec];
            [wd_distribution, wd_trans_matrix, wd_disc] = dataDistribution(weekdays,N_disc);
        else
            weekends = [weekends; loadvec];
            [we_distribution, we_trans_matrix, we_disc] = dataDistribution(weekends,N_disc);
        end
    end
    
    %% Increment daynum
    if daynum == 7
        daynum = 1;
    else
        daynum = daynum + 1;
    end
    
end % for day

%% Calculating global optimum
% fprintf('\n\nCalculating global optimum: ');

[totalcost, cost_opt, path_opt, P_bat_opt, SOC_opt] = ...
    optimumCharge(load2016(1:opt_days,:),ppv(1:opt_days,:),spot2016(1:opt_days,:),utilityprice,structure,P_bat_nom,eff_ch,eff_di,C_bat,SOC_max,SOC_min,delta_soc,firstday);

[P_bat_opt, SOC_opt] = pbatConversion(load2016(1:opt_days,:),P_bat_opt,SOC_opt,SOC_initial,SOC_min);


%% Billing calculations

% fprintf('\n\n- - - - - - - - - - - - - - - - - - - - - \n');
% fprintf('Load number: \t \t %i \n',loadnum);
% fprintf('Utility structure: \t %i \n',structure);
% fprintf('Optimization horizon: \t %i days \n',opt_days);
% fprintf('Load discretization: \t %i levels \n',N_disc);
% 
% if pv == 1
%     fprintf('PV: \t \t \t Yes \n');
%     fprintf('\n');
% else
%     fprintf('PV: \t \t \t No \n');
%     fprintf('\n');
% end

fprintf('- - - - - - - - - - - - - - - - - - - - - \n');
fprintf('Billing information \n');
fprintf('- - - - - - - - - - - - - - - - - - - - - \n');

bill_vec = zeros(1,4);
spotprice = spot2016(1:opt_days,:);

% No PV
p_grid = load2016(1:opt_days,:);
billing_basecase = billing(p_grid, spotprice, utilityprice, firstday, structure);
bill_vec(1) = billing_basecase;

fprintf('Basecase \t \t NOK %.1f \n', billing_basecase);

% With PV
if pv == 1
    p_grid_pv = load2016(1:opt_days,:)-ppv(1:opt_days,:);
    billing_pv = billing(p_grid_pv, spotprice, utilityprice, firstday, structure);

    bill_vec(2) = billing_pv;
    
    fprintf('PV \t \t \t NOK %.1f \n', billing_pv);
end

% With PV (if pv == 1) and battery
P_grid_sdp = load2016(1:opt_days,:)-ppv(1:opt_days,:)+P_bat_sdp;
billing_pv_batsdp = billing(P_grid_sdp,spotprice,utilityprice, firstday,structure);

bill_vec(3) = billing_pv_batsdp;

fprintf('SDP \t \t \t NOK %.1f \n', billing_pv_batsdp);

P_grid_opt = load2016(1:opt_days,:)-ppv(1:opt_days,:)+P_bat_opt;
billing_pv_batopt = billing(P_grid_opt,spotprice,utilityprice, firstday,structure);

bill_vec(4) = billing_pv_batopt;

fprintf('Global optimum \t \t NOK %.1f \n', billing_pv_batopt);

%% Saving

filename = ['data/results/new_load' num2str(loadnum) 'structure' num2str(structure) 'opt_days' num2str(opt_days) 'pv' num2str(pv) 'N_disc' num2str(N_disc) 'c_m' num2str(c_multi) 'p_m' num2str(p_multi) 'learning' num2str(learning)];

save(filename,'P_bat_sdp','P_grid_sdp','SOC_sdp','P_bat_opt','P_grid_opt','SOC_opt','loadnum','structure','opt_days','pv','N_disc','bill_vec','c_multi','p_multi');

% filename = 'data/results/stomodaccuracyload3learning0';
% 
% save(filename,'load_ind','trans_matrix_final','trans_prob','learning','loadnum');

end