function [totalcost, transitioncost, path, p_bat, SOC] = optimumCharge(p_load,p_pv,spotprice,utilityprice,structure,p_bat_max,eff_ch,eff_di,C_bat,SOC_max,SOC_min,delta_soc,firstday)
% Assuming p_load, p_pv and prices has the same dimensions

% tic

%% Key parameters
N_soc = 1/delta_soc;          % Number of possible SOCs
N_days = size(p_load,1);
N_periods = size(p_load,2);
N_tot_periods = N_days*N_periods;      % Total number of periods. 1 year = 8760

soc_max_change_ch = floor(p_bat_max*eff_ch*N_soc/(C_bat));   % Maximum steps of change of SOC during charge(integer)
soc_max_change_di = floor(p_bat_max*N_soc/(C_bat*eff_di));   % Maximum steps of change of SOC furing discharge (integer)

%% Initialising arrays
totalcost = inf(N_soc,N_tot_periods);               % Total minimum cost to finalise to last period
transitioncost = inf(N_soc,N_tot_periods,N_soc);    % Costs to get from SOC [1st dim] in period [2nd dim] to SOC [3rd dim] in period [2nd dim + 1]
path = zeros(N_soc,N_tot_periods);                  % Gives which row (i.e. which SOC) that one should move to in period [N_periods + 1] to min(costs)
p_bat = zeros(N_soc,N_tot_periods);                 % Optimum p_bat given an initial SOC.
SOC = inf(N_soc,N_tot_periods);                     % Optimum SOC given an initial SOC.

daynum = firstday;
% calculate lastday (because this is recursively calculated)

totalcost(1,N_tot_periods) = 0;             % Final state being SOC = 0.
transitioncost(1,N_tot_periods,:) = 0;      % Final cost = 0

soc_max_tp1 = 1;           % First row of SOC-array corresponding to soc_min
day = N_days;
t = N_periods;

weeks = N_days/7;

if mod(weeks,1) == 0 
    % Do nothing, as lastday = firstday
elseif weeks < 1
    daynum = firstday + N_days - 1; 
else
    weeks = floor(weeks);
    daynum = firstday + N_days - weeks*7 - 1;
end

if daynum > 7
    daynum = daynum - 7;
end

%% Calculations
for t = (N_tot_periods-1):-1:1
    
    %% Decide possible SOC
%     soc_max_t = min(N_soc,soc_max_tp1+soc_max_change);
    soc_max_t = min(N_soc,soc_max_tp1+soc_max_change_di);
    
    %% Set day and hour with respect to t
    if mod(t,24) == 0
        hour = 24;
        day = day - 1;
        daynum = daynum - 1;
        if daynum == 0
            daynum = 7;
        end
    else
        hour = mod(t,24);
    end
        
    %% Calculate costs
    for soc_t = 1:soc_max_t % Iterates over all legal soc's in period t
        
        soc_max_t_tp1 = min(soc_max_tp1,soc_t+soc_max_change_ch);
        soc_min_t_tp1 = max(1,soc_t-soc_max_change_di);
    
        for soc_tp1 = soc_min_t_tp1:soc_max_t_tp1 % Iterates over all legal SOC's in period t+1 which can be reached from SOC soc_t in period t
            
            SOC_change = (soc_tp1-soc_t)/N_soc;
            p_bat_temp = SOC_change*C_bat; % p_bat defined positive into battery
            
            if p_bat_temp > 0
                p_bat_temp = p_bat_temp/eff_ch;  % eff_ch < 1
            else
                p_bat_temp = p_bat_temp*eff_di;  % eff_di < 1
            end
            
            p_grid = p_load(day,hour) + p_bat_temp - p_pv(day,hour);
            if p_grid > 0
                switch structure
                    case 1
                        % Only spotprice
                        transitioncost(soc_t,t,soc_tp1) = spotprice(day,hour)*p_grid;
                        
                    case 2
                        % Flat rate                       
                        transitioncost(soc_t,t,soc_tp1) = (spotprice(day,hour) + utilityprice)*p_grid;  % > 0
                        
                    case 3
                        % Timely

                        % How to calculate the right daynum?
                        
                        transitioncost(soc_t,t,soc_tp1) = (spotprice(day,hour) + utilityprice(daynum,hour))*p_grid;  % > 0

                    case 4
                        % P-based
                        utilityprice_temp = utilityprice*p_grid;
                        transitioncost(soc_t,t,soc_tp1) = (spotprice(day,hour) + utilityprice_temp)*p_grid;
                        
                end % switch
            else
                transitioncost(soc_t,t,soc_tp1) = spotprice(day,hour)*p_grid; % < 0
            end % if p_grid > 0
            
        end % soc_tp1
        
        %% Calculate SOC-value and find shortest path
        change_cost = transitioncost(soc_t,t,:); % All transition costs for given period and soc
        change_cost = squeeze(change_cost);
        
        %SOC_temp = SOC(soc_min_j:soc_max_j,t+1) + change_cost;
        totalcost_temp = totalcost(:,t+1) + change_cost;
        
        [min_cost, min_place] = min(totalcost_temp); % Minimum cost and transition
        
        totalcost(soc_t, t) = min_cost;
        
        path(soc_t,t) = min_place;
        
    end % soc_t
    
    %% Preperation for new iteration
    soc_max_tp1 = soc_max_t;
end % t

%% Determining p_bat

for i = 1:N_soc
    soc_from = i;
    for j = 1:N_tot_periods
        
        soc_to = path(soc_from,j);
        
        p_bat_temp = (soc_to-soc_from)*C_bat/N_soc;
        
        if p_bat_temp > 0
            p_bat(i,j) = p_bat_temp/eff_ch;  % eff_ch < 0
        else
            p_bat(i,j) = p_bat_temp*eff_di;  % eff_di < 0
        end
        
        if soc_from == 1
            SOC(i,j) = 0;
        else
            SOC(i,j) = soc_from*(SOC_max-SOC_min)/N_soc;
        end
        
        soc_from = soc_to;
    end
end

p_bat(:,N_tot_periods) = 0;

% time = toc;

end