clear

%% Parameteres
loadnum = 3;
day = 27;
% daynum = 7;
structure = 4;
c_multi = 1;
p_multi = 1;

%% Finding daynum
weeks = day/7;

if mod(weeks,1) == 0 
    daynum = 4;
elseif weeks < 1
    daynum = 5 + day - 1; 
else
    weeks = floor(weeks);
    daynum = 5 + day - weeks*7 - 1;
end

if daynum > 7
    daynum = daynum - 7;
end

%% Load
load2016 = loadread(2016,loadnum);
loadvec = load2016(day,:);

%% Spot and utility
spot2016 = priceread(2016);
pricevec = spot2016(day,:);

utility = getUtilityPrice(2016,structure);

%% PV
ppvvec = zeros(1,24);

%% Battery info
[eff_ch,eff_di,C_bat,P_bat_nom,SOC_max,SOC_min] = batteryData(c_multi,p_multi); % This could be a input as a struct.

N_soc = 100;
delta_soc = 1/N_soc; % For optimiser

SOC_initial = 0;
SOC_final = 0;

soc_initial = SOC2socnum(SOC_initial,SOC_max,SOC_min,N_soc);
soc_final = SOC2socnum(SOC_final,SOC_max,SOC_min,N_soc);

%% Setting stochastic matrices

sto_filename = ['data/loaddisc' num2str(loadnum)];
load(sto_filename);

switch loadnum
    case 1
        if daynum <= 5
            trans_matrix = wd_trans_matrix1;
            load_disc = wd_disc1;
        else
            trans_matrix = we_trans_matrix1;
            load_disc = we_disc1;
        end
    case 2
        if daynum <= 5
            trans_matrix = wd_trans_matrix2;
            load_disc = wd_disc2;
        else
            trans_matrix = we_trans_matrix2;
            load_disc = we_disc2;
        end
    case 3
        if daynum <= 5
            trans_matrix = wd_trans_matrix3;
            load_disc = wd_disc3;
        else
            trans_matrix = we_trans_matrix3;
            load_disc = we_disc3;
        end
end

%% Global optimisation
% [totalcost, cost_opt, path_opt, P_bat_opt, SOC_opt] = ...
%     optimumCharge(loadvec,ppvvec,pricevec,utility,structure,P_bat_nom,eff_ch,eff_di,C_bat,SOC_max,SOC_min,delta_soc,daynum);
% 
% [P_bat_opt, SOC_opt] = pbatConversion(loadvec,P_bat_opt,SOC_opt,SOC_initial,SOC_min);

%% Stochastic optimisation
[path_fSDP, time] = fixedHorizonSDP(load_disc,trans_matrix,ppvvec,...
            pricevec,utility,structure,P_bat_nom,eff_ch,eff_di,...
            C_bat,SOC_max,SOC_min,N_soc,SOC_initial,SOC_final,daynum);

soc_t = zeros(1,24);
soc_t(1) = soc_initial;
soc_t(24) = soc_final;

load_ind_t = zeros(1,24);

trans_matrix_final = zeros(20,24);

for t = 1:24
    
    loadtemp_t = abs(load_disc(:,t)-loadvec(t));

    [~, load_ind_t(t)] = min(loadtemp_t);

    soc_t(t+1) = path_fSDP(soc_t(t),t,load_ind_t(t));
    
    trans_matrix_final(:,t+1) = trans_matrix(load_ind_t(t),t,:);
    
end

%% Plotting
figure('Position',[500 500 650 300])

hold on
% surf(trans_matrix_final,'EdgeColor','none')
contour(trans_matrix_final,50)
xlim([1 24])
ylim([1 20])
colormap(flipud(colormap))
% colorbar
c = contourcbar;
ylabel(c,'Transition probability','FontSize',11,'fontweight','bold')
set(c,'YTick',0:0.2:1);
caxis([0, 1])
set(gca,'xtick',1:1:24);
set(gca,'ytick',2:2:20);
view(0.1,-90)
ylabel('Load state','fontweight','bold')
xlabel('Hour','fontweight','bold')
ax = gca;
ax.XAxisLocation = 'top';

loadplot = plot3(1:24,load_ind_t,-1*ones(1,24),'k','LineWidth',2);
legend(loadplot,'Load path','location','northeast')
grid on
box on


% print(['/Users/erdal/Dropbox/Skole/NTNU/ShareLaTeX/NTNU Masters Thesis/fig/results/stochastics/load' num2str(loadnum) ...
%     'day' num2str(day) 'str' num2str(structure)],'-depsc')
